@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name$
-// @(#) $Id: DwmIpv6Address.cc 8388 2016-04-17 03:23:45Z dwm $
+// @(#) $Id: DwmIpv6Address.cc 9898 2017-12-30 06:16:42Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2004-2007
 //  All rights reserved.
@@ -52,7 +52,7 @@ extern "C" {
 #include "DwmWritable.hh"
 #include "DwmIpv6Address.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.12/src/DwmIpv6Address.cc 8388 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.13/src/DwmIpv6Address.cc 9898 $");
 
 namespace Dwm {
 
@@ -286,5 +286,16 @@ namespace Dwm {
       rc = BZ2_bzwrite(bzf, (void *)&_addr, sizeof(_addr));
     return(rc);
   }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  std::ostream & operator << (std::ostream & os, const Ipv6Address & addr)
+  {
+    if (os) {
+      os << (std::string)addr;
+    }
+    return os;
+  }
   
 }  // namespace Dwm
