@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name$
-// @(#) $Id: DwmSocket.cc 9014 2017-04-11 09:53:02Z dwm $
+// @(#) $Id: DwmSocket.cc 9890 2017-12-26 07:54:08Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 1998-2007, 2016
 //  All rights reserved.
@@ -42,6 +42,8 @@
 extern "C" {
   #include <fcntl.h>
   #include <sys/param.h>
+  #include <sys/socket.h>
+  #include <net/if.h>
 }
 
 #include <cstdlib>
@@ -53,7 +55,7 @@ extern "C" {
 #include "DwmSocket.hh"
 #include "DwmSvnTag.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.12/src/DwmSocket.cc 9014 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.13/src/DwmSocket.cc 9890 $");
 
 namespace Dwm {
 
@@ -140,6 +142,17 @@ namespace Dwm {
     return(rc);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  Socket Socket::Accept(struct sockaddr_in6 & addr)
+  {
+    Socket  rc;
+    socklen_t  addrLen = sizeof(addr);
+    rc._fd = accept(this->_fd, (struct sockaddr *)&addr, &addrLen);
+    return rc;
+  }
+
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -153,7 +166,21 @@ namespace Dwm {
     }
     return rc;
   }
-  
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  Socket Socket::Accept(Ipv6Address & addr, uint16_t & port)
+  {
+    struct sockaddr_in6  sockAddr;
+    Socket  rc = Accept(sockAddr);
+    if (rc._fd >= 0) {
+      addr = Ipv6Address(sockAddr.sin6_addr);
+      port = ntohs(sockAddr.sin6_port);
+    }
+    return rc;
+  }
+
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -169,6 +196,21 @@ namespace Dwm {
     return(rc);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Accept(Socket & s, struct sockaddr_in6 & addr)
+  {
+    bool       rc = false;
+    socklen_t  addrLen = sizeof(addr);
+    int        fd = accept(this->_fd, (struct sockaddr *)&addr, &addrLen);
+    if (fd >= 0) {
+      s.Attach(fd);
+      rc = true;
+    }
+    return rc;
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -183,6 +225,21 @@ namespace Dwm {
     }
     return rc;
   }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Accept(Socket & s, Ipv6Address & addr, uint16_t & port)
+  {
+    struct sockaddr_in6  inAddr;
+    memset(&inAddr, 0, sizeof(inAddr));
+    bool  rc = Accept(s, inAddr);
+    if (rc) {
+      addr = Ipv6Address(inAddr.sin6_addr);
+      port = ntohs(inAddr.sin6_port);
+    }
+    return rc;
+  }
   
   //--------------------------------------------------------------------------
   //!  
@@ -235,12 +292,29 @@ namespace Dwm {
     }
     else {
       Syslog(LOG_ERR, "bind(%d,%p,%d) failed: %m",
-             this->_fd,&addr, sizeof(addr));
+             this->_fd, &addr, sizeof(addr));
     }
     
     return(rc);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Bind(const struct sockaddr_in6 & addr)
+  {
+    bool  rc = false;
+    if (bind(this->_fd, (const struct sockaddr *)&addr, sizeof(addr)) == 0) {
+      rc = true;
+    }
+    else {
+      Syslog(LOG_ERR, "bind(%d,%p,%d) failed: %m",
+             this->_fd, &addr, sizeof(addr));
+    }
+    
+    return(rc);
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -256,7 +330,23 @@ namespace Dwm {
     sockAddr.sin_port = htons(port);
     return Bind(sockAddr);
   }
-  
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Bind(const Ipv6Address & addr, uint16_t port)
+  {
+    struct sockaddr_in6  sockAddr;
+    memset(&sockAddr, 0, sizeof(sockAddr));
+#ifndef __linux__
+    sockAddr.sin6_len = sizeof(sockAddr);
+#endif
+    sockAddr.sin6_family = AF_INET6;
+    sockAddr.sin6_addr = addr;
+    sockAddr.sin6_port = htons(port);
+    return Bind(sockAddr);
+  }
+
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -272,6 +362,21 @@ namespace Dwm {
     return(rc);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  ssize_t Socket::RecvFrom(void *buf, size_t len, int flags,
+                           struct sockaddr_in6 & from)
+  {
+    ssize_t  rc = -1;
+    if (this->_fd >= 0) {
+      socklen_t  fromLen = sizeof(from);
+      rc = recvfrom(this->_fd, buf, len, flags, (struct sockaddr *)&from,
+                    &fromLen);
+    }
+    return(rc);
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -287,6 +392,21 @@ namespace Dwm {
     return rc;
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  ssize_t Socket::RecvFrom(void *buf, size_t len, int flags,
+                           Ipv6Address & srcAddr, uint16_t & srcPort)
+  {
+    struct sockaddr_in6  fromAddr;
+    ssize_t  rc = RecvFrom(buf, len, flags, fromAddr);
+    if (rc >= 0) {
+      srcAddr = fromAddr.sin6_addr;
+      srcPort = ntohs(fromAddr.sin6_port);
+    }
+    return rc;
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -301,6 +421,21 @@ namespace Dwm {
     }
     return rc;
   }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  ssize_t Socket::RecvFrom(std::string & s, int flags,
+                           Ipv6Address & srcAddr, uint16_t & srcPort)
+  {
+    s.clear();
+    char     buf[IP_MAXPACKET];
+    ssize_t  rc = RecvFrom(buf, IP_MAXPACKET, flags, srcAddr, srcPort);
+    if (rc > 0) {
+      s.assign(buf, rc);
+    }
+    return rc;
+  }
   
   //--------------------------------------------------------------------------
   //!  
@@ -323,6 +458,20 @@ namespace Dwm {
     return(rc);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  ssize_t Socket::SendTo(const void *buf, size_t len, int flags,
+                         const struct sockaddr_in6 & to)
+  {
+    ssize_t  rc = -1;
+    if (this->_fd >= 0) {
+      socklen_t  toLen = sizeof(to);
+      rc = sendto(this->_fd, buf, len, flags, (struct sockaddr *)&to, toLen);
+    }
+    return(rc);
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -340,6 +489,23 @@ namespace Dwm {
     return SendTo(buf, len, flags, addr);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  ssize_t Socket::SendTo(const void *buf, size_t len, int flags,
+                         const Ipv6Address & dstAddr, uint16_t dstPort)
+  {
+    sockaddr_in6  addr;
+    memset(&addr, 0, sizeof(addr));
+#ifndef __linux__
+    addr.sin6_len = sizeof(addr);
+#endif
+    addr.sin6_family = AF_INET6;
+    addr.sin6_addr = dstAddr;
+    addr.sin6_port = htons(dstPort);
+    return SendTo(buf, len, flags, addr);
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -348,6 +514,15 @@ namespace Dwm {
   {
     return SendTo(s.data(), s.size(), flags, dstAddr, dstPort);
   }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  ssize_t Socket::SendTo(const std::string & s, int flags,
+                         const Ipv6Address & dstAddr, uint16_t dstPort)
+  {
+    return SendTo(s.data(), s.size(), flags, dstAddr, dstPort);
+  }
   
   //--------------------------------------------------------------------------
   //!  
@@ -372,6 +547,17 @@ namespace Dwm {
     return(rc);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Connect(const struct sockaddr_in6 & name)
+  {
+    bool  rc = false;
+    if (connect(this->_fd, (const struct sockaddr *)&name, sizeof(name)) == 0)
+      rc = true;
+    return(rc);
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -384,6 +570,19 @@ namespace Dwm {
     addr.sin_port = htons(port);
     return Connect(addr);
   }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Connect(const Ipv6Address & dstAddr, uint16_t port)
+  {
+    struct sockaddr_in6  addr;
+    memset(&addr, 0, sizeof(addr));
+    addr.sin6_family = AF_INET6;
+    addr.sin6_addr = dstAddr;
+    addr.sin6_port = htons(port);
+    return Connect(addr);
+  }
   
   //--------------------------------------------------------------------------
   //!  
@@ -424,6 +623,24 @@ namespace Dwm {
     }
     return rc;
   }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Getpeername(Ipv6Address & addr, uint16_t & port)
+  {
+    bool  rc = false;
+    struct sockaddr_in6  sockAddr;
+    socklen_t            sockLen = sizeof(sockAddr);
+    if (Getpeername((struct sockaddr *)&sockAddr, sockLen)) {
+      if (sockAddr.sin6_family == AF_INET6) {
+        addr = Ipv6Address(sockAddr.sin6_addr);
+        port = ntohs(sockAddr.sin6_port);
+        rc = true;
+      }
+    }
+    return rc;
+  }
   
   //--------------------------------------------------------------------------
   //!  
@@ -454,6 +671,24 @@ namespace Dwm {
     return rc;
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::Getsockname(Ipv6Address & addr, uint16_t & port)
+  {
+    bool  rc = false;
+    struct sockaddr_in6  sockAddr;
+    socklen_t            sockLen = sizeof(sockAddr);
+    if (Getsockname((struct sockaddr *)&sockAddr, sockLen)) {
+      if (sockAddr.sin6_family == AF_INET6) {
+        addr = Ipv6Address(sockAddr.sin6_addr);
+        port = ntohs(sockAddr.sin6_port);
+        rc = true;
+      }
+    }
+    return rc;
+  }
+  
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
@@ -494,6 +729,35 @@ namespace Dwm {
     mreq.imr_interface.s_addr = intfAddr.Raw();
     return Setsockopt(IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq));
   }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::JoinMulticastGroup(const Ipv6Address & groupAddr,
+                                  unsigned int ifIndex)
+  {
+    bool  rc = false;
+    struct ipv6_mreq  mreq;
+    mreq.ipv6mr_multiaddr = groupAddr;
+    mreq.ipv6mr_interface = ifIndex;
+    return Setsockopt(IPPROTO_IPV6, IPV6_JOIN_GROUP, &mreq, sizeof(mreq));
+  }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Socket::JoinMulticastGroup(const Ipv6Address & groupAddr,
+                                  const std::string & ifName)
+  {
+    bool  rc = false;
+    if (! ifName.empty()) {
+      unsigned int  ifIndex = if_nametoindex(ifName.c_str());
+      if (ifIndex > 0) {
+        rc = JoinMulticastGroup(groupAddr, ifIndex);
+      }
+    }
+    return rc;
+  }
   
   //--------------------------------------------------------------------------
   //!  
