@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.12/src/DwmSunriseSunset.cc 9014 $
-// @(#) $Id: DwmSunriseSunset.cc 9014 2017-04-11 09:53:02Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.13/src/DwmSunriseSunset.cc 9938 $
+// @(#) $Id: DwmSunriseSunset.cc 9938 2018-01-05 23:37:32Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2016
 //  All rights reserved.
@@ -39,8 +39,9 @@
 //!  \brief Sunrise, Sunset and IsDaytime implementations
 //---------------------------------------------------------------------------
 
-//  Deal with linux stupidity with respect to isnan()
+//  Deal with linux silliness with respect to isnan()
 #ifdef __linux__
+  #define _DEFAULT_SOURCE
   #define _BSD_SOURCE
   extern "C" {
     #include <math.h>
@@ -52,7 +53,7 @@
 #include "DwmSunriseSunset.hh"
 #include "DwmSvnTag.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.12/src/DwmSunriseSunset.cc 9014 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.13/src/DwmSunriseSunset.cc 9938 $");
 
 namespace Dwm {
 
