@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.12/tests/TestDwmSocket.cc 9015 $
-// @(#) $Id: TestDwmSocket.cc 9015 2017-04-11 16:50:27Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.13/tests/TestDwmSocket.cc 9892 $
+// @(#) $Id: TestDwmSocket.cc 9892 2017-12-26 08:16:56Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2016
 //  All rights reserved.
@@ -45,7 +45,8 @@ extern "C" {
   #include <netinet/in.h>
   #include <netinet/tcp.h>
 }
-  
+
+#include <atomic>
 #include <thread>
 
 #include "DwmSocket.hh"
@@ -53,7 +54,7 @@ extern "C" {
 #include "DwmSysLogger.hh"
 #include "DwmUnitAssert.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.12/tests/TestDwmSocket.cc 9015 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.6.13/tests/TestDwmSocket.cc 9892 $");
 
 using namespace std;
 using namespace Dwm;
@@ -80,6 +81,8 @@ static void UdpClientThread()
   return;
 }
 
+atomic<bool>  g_udpServerReady(false);
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -88,6 +91,7 @@ static void UdpServerThread()
   Socket  s;
   if (UnitAssert(s.Open(PF_INET, SOCK_DGRAM, 0))) {
     if (UnitAssert(s.Bind(Ipv4Address("127.0.0.1"), 2190))) {
+      g_udpServerReady = true;
       string  hello("Hello"), sr;
       Ipv4Address  srcAddr;
       uint16_t     srcPort;
@@ -104,16 +108,71 @@ static void UdpServerThread()
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void Udp6ClientThread()
+{
+  Socket  s;
+  if (UnitAssert(s.Open(PF_INET6, SOCK_DGRAM, 0))) {
+    if (UnitAssert(s.SendTo("Hello", 0, Ipv6Address("::1"), 2190))) {
+      string  goodbye("Goodbye"), sr;
+      Ipv6Address  srcAddr;
+      uint16_t     srcPort;
+      if (UnitAssert(s.RecvFrom(sr, 0, srcAddr, srcPort) == goodbye.size())) {
+        UnitAssert(sr == goodbye);
+        UnitAssert(srcAddr == Ipv6Address("::1"));
+        UnitAssert(srcPort == 2190);
+      }
+    }
+    UnitAssert(s.Close());
+  }
+  return;
+}
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void Udp6ServerThread()
+{
+  Socket  s;
+  if (UnitAssert(s.Open(PF_INET6, SOCK_DGRAM, 0))) {
+    if (UnitAssert(s.Bind(Ipv6Address("::1"), 2190))) {
+      g_udpServerReady = true;
+      string  hello("Hello"), sr;
+      Ipv6Address  srcAddr;
+      uint16_t     srcPort;
+      ssize_t      bytesReceived = s.RecvFrom(sr, 0, srcAddr, srcPort);
+      if (UnitAssert(bytesReceived == hello.size())) {
+        UnitAssert(sr == hello);
+        UnitAssert(srcAddr == Ipv6Address("::1"));
+        string  goodbye("Goodbye");
+        UnitAssert(s.SendTo(goodbye, 0, srcAddr, srcPort) == goodbye.size());
+      }
+    }
+    UnitAssert(s.Close());
+  }
+  return;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
 static void TestUdpClientServer()
 {
+  g_udpServerReady = false;
   thread  serverThread(UdpServerThread);
-  sleep(1);
+  while (! g_udpServerReady) { }
   thread  clientThread(UdpClientThread);
   clientThread.join();
   serverThread.join();
+
+  g_udpServerReady = false;
+  thread  server6Thread(Udp6ServerThread);
+  while(! g_udpServerReady) { }
+  thread  client6Thread(Udp6ClientThread);
+  client6Thread.join();
+  server6Thread.join();
   return;
 }
 
@@ -137,6 +196,8 @@ static void TcpClientThread()
   return;
 }
 
+static atomic<bool>  g_tcpServerReady(false);
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -149,6 +210,7 @@ static void TcpServerThread()
                             &on, sizeof(on)));
     if (UnitAssert(s.Bind(Ipv4Address("127.0.0.1"), 2190))) {
       if (UnitAssert(s.Listen(10))) {
+        g_tcpServerReady = true;
         Ipv4Address  peerAddr;
         uint16_t     peerPort;
         Socket  sa = s.Accept(peerAddr, peerPort);
@@ -181,16 +243,89 @@ static void TcpServerThread()
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void Tcp6ClientThread()
+{
+  Socket  s;
+  if (UnitAssert(s.Open(PF_INET6, SOCK_STREAM, 0))) {
+    if (UnitAssert(s.Connect(Ipv6Address("::1"), 2190))) {
+      int  on = 1;
+      UnitAssert(s.Setsockopt(IPPROTO_TCP, TCP_NODELAY,
+                              &on, sizeof(on)));
+      UnitAssert(write(s, "Hello", strlen("Hello")) == strlen("Hello"));
+      char  buf[65535] = { 0 };
+      UnitAssert(read(s, buf, sizeof(buf)) == strlen("Goodbye"));
+    }
+    UnitAssert(s.Close());
+  }
+  return;
+}
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void Tcp6ServerThread()
+{
+  Socket  s;
+  if (UnitAssert(s.Open(PF_INET6, SOCK_STREAM, 0))) {
+    int  on = 1;
+    UnitAssert(s.Setsockopt(SOL_SOCKET, SO_REUSEADDR,
+                            &on, sizeof(on)));
+    if (UnitAssert(s.Bind(Ipv6Address("::1"), 2190))) {
+      if (UnitAssert(s.Listen(10))) {
+        g_tcpServerReady = true;
+        Ipv6Address  peerAddr;
+        uint16_t     peerPort;
+        Socket  sa = s.Accept(peerAddr, peerPort);
+        if (UnitAssert((int)sa >= 0)) {
+          Ipv6Address  myAddr;
+          uint16_t     myPort;
+          UnitAssert(sa.Getsockname(myAddr, myPort));
+          UnitAssert(myAddr == Ipv6Address("::1"));
+          UnitAssert(myPort == 2190);
+          UnitAssert(peerAddr == Ipv6Address("::1"));
+          UnitAssert(peerPort > 1024);
+          Ipv6Address  peerAddr2;
+          uint16_t     peerPort2;
+          UnitAssert(sa.Getpeername(peerAddr2, peerPort2));
+          UnitAssert(peerAddr2 == peerAddr);
+          UnitAssert(peerPort2 == peerPort);
+          int  noDelay = 1;
+          UnitAssert(sa.Setsockopt(IPPROTO_TCP, TCP_NODELAY,
+                                   &noDelay, sizeof(noDelay)));
+          char  buf[65535] = { 0 };
+          UnitAssert(read(sa, buf, sizeof(buf)) == strlen("Hello"));
+          UnitAssert(write(sa, "Goodbye", strlen("Goodbye"))
+                     == strlen("Goodbye"));
+          UnitAssert(sa.Close());
+        }
+      }
+    }
+    UnitAssert(s.Close());
+  }
+  return;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
 static void TestTcpClientServer()
 {
+  g_tcpServerReady = false;
   thread  serverThread(TcpServerThread);
-  sleep(1);
+  while (! g_tcpServerReady) { }
   thread  clientThread(TcpClientThread);
   clientThread.join();
   serverThread.join();
+
+  g_tcpServerReady = false;
+  thread  server6Thread(Tcp6ServerThread);
+  while(! g_tcpServerReady) { }
+  thread  client6Thread(Tcp6ClientThread);
+  client6Thread.join();
+  server6Thread.join();
   return;
 }
 
@@ -239,6 +374,52 @@ static void MulticastClient(const string & sendStr,
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void Multicast6Client(const string & sendStr,
+                            const string & expectedString)
+{
+  Socket  recvSock;
+  struct in6_addr  in6Addr = IN6ADDR_ANY_INIT;
+  Ipv6Address  bindAddr(in6Addr);
+  Ipv6Address  mcastAddr("ff15::1");
+  string       intfName("lo0");
+  
+  if (UnitAssert(recvSock.Open(PF_INET6, SOCK_DGRAM, IPPROTO_UDP))) {
+    int on = 1;
+    recvSock.Setsockopt(SOL_SOCKET, SO_REUSEPORT, &on, sizeof(on));
+    if (UnitAssert(recvSock.Bind(bindAddr, 2191))) {
+      if (UnitAssert(recvSock.JoinMulticastGroup(mcastAddr, intfName))) {
+        Socket  sendSock;
+        if (UnitAssert(sendSock.Open(PF_INET6, SOCK_DGRAM, 0))) {
+          int   tries = 0;
+          bool  otherSeen = false;
+          while (tries < 20) {
+            if (UnitAssert(sendSock.SendTo(sendStr, 0, mcastAddr, 2191))) {
+              string       rs;
+              Ipv6Address  srcAddr;
+              uint16_t     srcPort;
+              if (! recvSock.ReadWouldBlock()) {
+                UnitAssert(recvSock.RecvFrom(rs, 0, srcAddr, srcPort) > 0);
+                if (rs == expectedString) {
+                  otherSeen = true;
+                  break;
+                }
+                ++tries;
+              }
+            }
+          }
+          UnitAssert(otherSeen);
+          sendSock.Close();
+        }
+      }
+    }
+  }
+  recvSock.Close();
+  return;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -250,6 +431,13 @@ static void TestMulticast()
                   "sent by first client");
   client1.join();
   client2.join();
+
+  thread  client1_6(Multicast6Client, "sent by first client",
+                  "sent by second client");
+  thread  client2_6(Multicast6Client, "sent by second client",
+                    "sent by first client");
+  client1_6.join();
+  client2_6.join();
 }
 
 //----------------------------------------------------------------------------
